@@ -53,6 +53,7 @@
 
 #include "parsejob.h"
 #include "parserdependencypolicy.h"
+#include <editor/modificationrevisionset.h>
 
 namespace KDevelop
 {
@@ -63,6 +64,8 @@ public:
     BackgroundParserPrivate(BackgroundParser *parser, ILanguageController *languageController)
         :m_parser(parser), m_languageController(languageController)
     {
+        parser->d = this; //Set this so we can safely call back BackgroundParser from within loadSettings()
+        
         m_timer.setSingleShot(true);
         m_delay = 500;
         m_threads = 1;
@@ -76,6 +79,11 @@ public:
 
         loadSettings(); // Start the weaver
     }
+    
+    void startTimerThreadSafe() {
+        QMetaObject::invokeMethod(m_parser, "startTimer", Qt::QueuedConnection);
+    }
+    
     ~BackgroundParserPrivate()
     {
         suspend();
@@ -101,20 +109,28 @@ public:
         // Create delayed jobs, that is, jobs for documents which have been changed
         // by the user.
         QList<ParseJob*> jobs;
-        QHashIterator<KUrl, DocumentChangeTracker*> it = m_delayedParseJobs;
-        while (it.hasNext()) {
-            ParseJob* job = createParseJob(it.next().key(), 
-                                           TopDUContext::AllDeclarationsContextsAndUses,
-                                           QList<QPointer<QObject> >());
-            if (job) {
-                job->setChangedRanges(it.value()->changedRanges());
-                jobs.append(job);
-            } else {
-                kWarning() << "No job created for url " << it.key();
+        
+        for(QHash<KUrl, DocumentChangeTracker*>::iterator it = m_delayedParseJobs.begin(); it != m_delayedParseJobs.end(); ) {
+            KUrl url(it.key());
+            
+            if(m_parseJobs.contains(url)) {
+                kDebug() << "already parsing" << url << ", delaying the parse-job";
+                ++it; //Add the delayed job later
+            }else{
+                ParseJob* job = createParseJob(url, 
+                                            TopDUContext::AllDeclarationsContextsAndUses,
+                                            QList<QPointer<QObject> >());
+                if (job) {
+                    job->setChangedRanges(it.value()->changedRanges());
+                    jobs.append(job);
+                    specialParseJob = job;
+                } else {
+                    kWarning() << "No job created for url " << it.key();
+                }
+                delete *it;
+                it = m_delayedParseJobs.erase(it);
             }
         }
-        qDeleteAll(m_delayedParseJobs);
-        m_delayedParseJobs.clear();
 
         for (QMap<int, QSet<KUrl> >::Iterator it1 = m_documentsForPriority.begin();
              it1 != m_documentsForPriority.end(); ++it1 )
@@ -124,18 +140,25 @@ public:
             
             for(QSet<KUrl>::Iterator it = it1.value().begin(); it != it1.value().end();) {
                 //Only create parse-jobs for up to thread-count * 2 documents, so we don't fill the memory unnecessarily
-                if(this->m_parseJobs.size() + jobs.size() > (m_threads*2)+1)
+                if(m_parseJobs.count() >= m_threads+1)
                     break;
                 
+                if(m_parseJobs.count() >= m_threads && it1.key() > BackgroundParser::NormalPriority && !specialParseJob)
+                    break; //The additional parsing thread is reserved for higher priority parsing
+                
                 // When a document is scheduled for parsing while it is being parsed, it will be parsed
                 // again once the job finished, but not now.
                 if (m_parseJobs.contains(*it) ) {
                     ++it;
                     continue;
                 }
-
-                kDebug(9505) << "creating parse-job" << *it;
+                
+                kDebug(9505) << "creating parse-job" << *it << "new count of active parse-jobs:" << m_parseJobs.count() + 1;
                 ParseJob* job = createParseJob(*it, m_documents[*it].features(), m_documents[*it].notifyWhenReady());
+                
+                if(m_parseJobs.count() == m_threads+1 && !specialParseJob)
+                    specialParseJob = job; //This parse-job is allocated into the reserved thread
+                
                 if(job)
                     jobs.append(job);
                 
@@ -154,7 +177,7 @@ public:
 
         //We don't hide the progress-bar in updateProgressBar, so it doesn't permanently flash when a document is reparsed again and again.
         if(m_doneParseJobs == m_maxParseJobs)
-            emit m_parser->hideProgress();
+            emit m_parser->hideProgress(m_parser);
     }
 
     ParseJob* createParseJob(const KUrl& url, TopDUContext::Features features, QList<QPointer<QObject> > notifyWhenReady)
@@ -194,12 +217,13 @@ public:
 
     void loadSettings()
     {
+        ///@todo re-load settings when they have been changed!
         KConfigGroup config(KGlobal::config(), "Background Parser");
 
         m_delay = config.readEntry("Delay", 500);
         m_timer.setInterval(m_delay);
-        m_threads = config.readEntry("Real Number of Threads", 1);
-        m_weaver.setMaximumNumberOfThreads(m_threads);
+        m_threads = 0;
+        m_parser->setThreadCount(config.readEntry("Number of Threads", 1));
 
         if (config.readEntry("Enabled", true)) {
             resume();
@@ -237,6 +261,9 @@ public:
     BackgroundParser *m_parser;
     ILanguageController* m_languageController;
 
+    //Current parse-job that is executed in the additional thread
+    QPointer<ParseJob> specialParseJob;
+    
     QTimer m_timer;
     int m_delay;
     int m_threads;
@@ -411,9 +438,7 @@ void BackgroundParser::addDocument(const KUrl& url, TopDUContext::Features featu
 //             kDebug(9505) << "BackgroundParser::addDocument: is already queued:" << url;
         }
 
-        if (!d->m_timer.isActive()) {
-            d->m_timer.start();
-        }
+        d->startTimerThreadSafe();
     }
 }
 
@@ -481,15 +506,14 @@ void BackgroundParser::setNeededPriority(int priority)
 {
     QMutexLocker lock(&d->m_mutex);
     d->m_neededPriority = priority;
-    if(!d->m_timer.isActive())
-        d->m_timer.start();
+    d->startTimerThreadSafe();
 }
 
 void BackgroundParser::suspend()
 {
     d->suspend();
 
-    emit hideProgress();
+    emit hideProgress(this);
 }
 
 void BackgroundParser::resume()
@@ -511,7 +535,7 @@ void BackgroundParser::updateProgressBar()
         for(QMap<KDevelop::ParseJob*, float>::const_iterator it = d->m_jobProgress.constBegin(); it != d->m_jobProgress.constEnd(); ++it)
             additionalProgress += *it;
         
-        emit showProgress(0, d->m_maxParseJobs*1000, (additionalProgress + d->m_doneParseJobs)*1000);
+        emit showProgress(this, 0, d->m_maxParseJobs*1000, (additionalProgress + d->m_doneParseJobs)*1000);
     }
 }
  
@@ -535,7 +559,7 @@ void BackgroundParser::setThreadCount(int threadCount)
 {
     if (d->m_threads != threadCount) {
         d->m_threads = threadCount;
-        d->m_weaver.setMaximumNumberOfThreads(d->m_threads);
+        d->m_weaver.setMaximumNumberOfThreads(d->m_threads+1); //1 Additional thread for high-priority parsing
     }
 }
 
@@ -566,6 +590,10 @@ void BackgroundParser::rangeContentsChanged(KTextEditor::SmartRange* range, KTex
     // Smart mutex is already locked
     KUrl documentUrl = range->document()->url();
 
+    ///@todo Also detect these changes on-disk and do clearing
+    ModificationRevisionSet::clearCache();
+    KDevelop::ModificationRevision::clearModificationCache(IndexedString(documentUrl));
+
     if (d->m_parseJobs.contains(documentUrl)) {
         ParseJob* job = d->m_parseJobs[documentUrl];
         if (job->addChangedRange( mostSpecificChild ))
@@ -586,10 +614,15 @@ void BackgroundParser::rangeContentsChanged(KTextEditor::SmartRange* range, KTex
 
     newTracker->addChangedRange( mostSpecificChild );
     
+    d->startTimerThreadSafe();
+}
+
+void BackgroundParser::startTimer() {
     if (!d->m_timer.isActive())
         d->m_timer.start(d->m_delay);
 }
 
+
 }
 
 #include "backgroundparser.moc"
