@@ -164,7 +164,7 @@ Declaration::~Declaration()
   // always delete the declaration, to not create crashes within more complex code like C++ template stuff.
   if (context() && !d_func()->m_anonymousInContext) {
     if(!topContext->deleting() || !topContext->isOnDisk() || context()->d_func()->isDynamic())
-      Q_ASSERT(context()->m_dynamicData->removeDeclaration(this));
+      context()->m_dynamicData->removeDeclaration(this);
   }
     
   clearOwnIndex();
@@ -221,10 +221,9 @@ LocalIndexedDeclaration::LocalIndexedDeclaration(uint declarationIndex) : m_decl
 }
 
 Declaration* LocalIndexedDeclaration::data(TopDUContext* top) const {
-  if(m_declarationIndex)
-    return top->m_dynamicData->getDeclarationForIndex(m_declarationIndex);
-  else
+  if(!m_declarationIndex)
     return 0;
+  return top->m_dynamicData->getDeclarationForIndex(m_declarationIndex);
 }
 
 bool LocalIndexedDeclaration::isLoaded(TopDUContext* top) const {
@@ -250,7 +249,7 @@ IndexedDeclaration::IndexedDeclaration(Declaration* decl) {
 Declaration* IndexedDeclaration::declaration() const {
   if(isDummy())
     return 0;
-  ENSURE_CHAIN_READ_LOCKED
+//   ENSURE_CHAIN_READ_LOCKED
   if(!m_topContext || !m_declarationIndex)
     return 0;
 
@@ -349,12 +348,18 @@ DUContext * Declaration::context() const
   return m_context;
 }
 
+bool Declaration::isAnonymous() const
+{
+  return d_func()->m_anonymousInContext;
+}
+
 void Declaration::setContext(DUContext* context, bool anonymous)
 {
   Q_ASSERT(!context || context->topContext());
   ///@todo re-enable. In C++ support we need a short window to put visible declarations into template contexts
   if(!specialization()) {
-    ENSURE_CAN_WRITE
+    //problem: specialization() doesn't work during destructor
+//     ENSURE_CAN_WRITE
   }
 
   setInSymbolTable(false);
@@ -474,7 +479,7 @@ DUContext * Declaration::logicalInternalContext(const TopDUContext* topContext)
 
 DUContext * Declaration::internalContext() const
 {
-  ENSURE_CAN_READ
+//   ENSURE_CAN_READ
   return d_func()->m_internalContext.context();
 }
 
