@@ -24,16 +24,11 @@
 #include "declaration.h"
 #include "persistentsymboltable.h"
 #include "arrayhelpers.h"
+#include "instantiationinformation.h"
 #include <util/convenientfreelist.h>
 
 namespace KDevelop {
 
-struct DeclarationTopContextExtractor {
-  static IndexedTopDUContext extract(const IndexedDeclaration& decl) {
-    return decl.indexedTopContext();
-  }
-};
-
 DeclarationId::DeclarationId(const IndexedQualifiedIdentifier& id, uint additionalId, uint specialization)
   : m_direct(false), m_specialization(specialization)
 {
@@ -79,12 +74,9 @@ KDevVarLengthArray<Declaration*> DeclarationId::getDeclarations(const TopDUConte
     //Find the declaration by its qualified identifier and additionalIdentity
     QualifiedIdentifier id(indirect.m_identifier);
 
-    PersistentSymbolTable::Declarations decls = PersistentSymbolTable::self().getDeclarations(id);
-    
     if(top) {
       //Do filtering
-      ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 TopDUContext::IndexedRecursiveImports, DeclarationTopContextExtractor> filter(decls.iterator(), top->recursiveImportIndices());
+      PersistentSymbolTable::FilteredDeclarationIterator filter = PersistentSymbolTable::self().getFilteredDeclarations(id, top->recursiveImportIndices());
       for(; filter; ++filter) {
           Declaration* decl = filter->data();
           if(decl && indirect.m_additionalIdentity == decl->additionalIdentity()) {
@@ -94,6 +86,7 @@ KDevVarLengthArray<Declaration*> DeclarationId::getDeclarations(const TopDUConte
       }
     }else{
       //Just accept anything
+      PersistentSymbolTable::Declarations decls = PersistentSymbolTable::self().getDeclarations(id);
       PersistentSymbolTable::Declarations::Iterator decl = decls.iterator();
       for(; decl; ++decl) {
         const IndexedDeclaration& iDecl(*decl);
@@ -136,12 +129,9 @@ Declaration* DeclarationId::getDeclaration(const TopDUContext* top) const
     //Find the declaration by its qualified identifier and additionalIdentity
     QualifiedIdentifier id(indirect.m_identifier);
 
-    PersistentSymbolTable::Declarations decls = PersistentSymbolTable::self().getDeclarations(id);
-    
     if(top) {
       //Do filtering
-      ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 TopDUContext::IndexedRecursiveImports, DeclarationTopContextExtractor> filter(decls.iterator(), top->recursiveImportIndices());
+      PersistentSymbolTable::FilteredDeclarationIterator filter = PersistentSymbolTable::self().getFilteredDeclarations(id, top->recursiveImportIndices());
       for(; filter; ++filter) {
           Declaration* decl = filter->data();
           if(decl && indirect.m_additionalIdentity == decl->additionalIdentity()) {
@@ -153,6 +143,7 @@ Declaration* DeclarationId::getDeclaration(const TopDUContext* top) const
       }
     }else{
       //Just accept anything
+      PersistentSymbolTable::Declarations decls = PersistentSymbolTable::self().getDeclarations(id);
       PersistentSymbolTable::Declarations::Iterator decl = decls.iterator();
       for(; decl; ++decl) {
         const IndexedDeclaration& iDecl(*decl);
@@ -185,13 +176,22 @@ Declaration* DeclarationId::getDeclaration(const TopDUContext* top) const
 }
 
 QualifiedIdentifier DeclarationId::qualifiedIdentifier() const {
-  Declaration* decl = getDeclaration(0);
-  if(decl)
-    return decl->qualifiedIdentifier();
-  else if(m_direct)
+  
+  if(!m_direct) {
+    QualifiedIdentifier baseIdentifier = indirect.m_identifier.identifier();
+    if(!m_specialization)
+      return baseIdentifier;
+    ///@todo Fix m_specialization to IndexedInstantiationInformation for security
+    return IndexedInstantiationInformation(m_specialization).information().applyToIdentifier(baseIdentifier);
+  } else {
+    Declaration* decl = getDeclaration(0);
+    if(decl)
+      return decl->qualifiedIdentifier();    
+    
     return QualifiedIdentifier(QString("(unknown direct declaration)"));
-  else
-    return QualifiedIdentifier(QString("(missing)")) + indirect.m_identifier.identifier();
+  }
+  
+  return QualifiedIdentifier(QString("(missing)")) + indirect.m_identifier.identifier();
 }
 
 }
