@@ -174,7 +174,7 @@ DUContext* LocalIndexedDUContext::data(TopDUContext* top) const {
 DUContext* IndexedDUContext::context() const {
   if(isDummy())
     return 0;
-  ENSURE_CHAIN_READ_LOCKED
+//   ENSURE_CHAIN_READ_LOCKED
   if(!m_topContext)
     return 0;
 
@@ -579,7 +579,10 @@ DUContext::~DUContext( )
 
   deleteChildContextsRecursively();
 
-  deleteUses();
+  if(!topContext()->deleting() || !topContext()->isOnDisk())
+    deleteUses();
+  else
+    clearUseSmartRanges();
 
   deleteLocalDeclarations();
 
@@ -589,10 +592,10 @@ DUContext::~DUContext( )
     if (m_dynamicData->m_parentContext)
       m_dynamicData->m_parentContext->m_dynamicData->removeChildContext(this);
     //DUChain::contextChanged(this, DUChainObserver::Deletion, DUChainObserver::NotApplicable);
-
-    top->m_dynamicData->clearContextIndex(this);
   }
   
+  top->m_dynamicData->clearContextIndex(this);
+  
   Q_ASSERT(d_func()->isDynamic() == (!top->deleting() || !top->isOnDisk() || top->m_dynamicData->isTemporaryContextIndex(m_dynamicData->m_indexInTopContext)));
 }
 
@@ -1235,15 +1238,31 @@ void DUContext::deleteUse(int index)
   }
 }
 
+QVector<KTextEditor::SmartRange*> DUContext::takeUseRanges()
+{
+  ENSURE_CAN_WRITE
+  QVector<KTextEditor::SmartRange*> ret = m_dynamicData->m_rangesForUses;
+  
+  foreach(KTextEditor::SmartRange* range, ret)
+    if(range)
+      range->removeWatcher(this);
+    
+  m_dynamicData->m_rangesForUses.clear();
+  return ret;
+}
+
+QVector<KTextEditor::SmartRange*> DUContext::useRanges()
+{
+  ENSURE_CAN_READ
+  return m_dynamicData->m_rangesForUses;
+}
+
 void DUContext::deleteUses()
 {
   ENSURE_CAN_WRITE
   
-  if(!topContext()->deleting() || !topContext()->isOnDisk())
-  {
-    DUCHAIN_D_DYNAMIC(DUContext);
-    d->m_usesList().clear();
-  }
+  DUCHAIN_D_DYNAMIC(DUContext);
+  d->m_usesList().clear();
 
   clearUseSmartRanges();
 }
@@ -1423,7 +1442,8 @@ void DUContext::findContextsInternal(ContextType contextType, const SearchItem::
 
 bool DUContext::shouldSearchInParent(SearchFlags flags) const
 {
-  return !(flags & InImportedParentContext);
+  return (parentContext() && parentContext()->type() == DUContext::Helper && (flags & InImportedParentContext)) ||
+         !(flags & InImportedParentContext);
 }
 
 const Use* DUContext::uses() const
@@ -1625,10 +1645,13 @@ void DUContext::clearImportedParentContexts()
   DUCHAIN_D_DYNAMIC(DUContext);
 
   while( d->m_importedContextsSize() != 0 ) {
-    if( d->m_importedContexts()[0].isBackwardMapped() && d->m_importedContexts()[0].context(0) )
-      removeImportedParentContext(d->m_importedContexts()[0].context(0));
-    else
-      d->m_importedContextsList().removeOne(d->m_importedContexts()[0]);
+    if( d->m_importedContexts()[0].isBackwardMapped() ) {
+      DUContext* ctx = d->m_importedContexts()[0].context(0);
+      if(ctx)
+        ctx->m_dynamicData->removeImportedChildContext(this);
+    }
+    
+    d->m_importedContextsList().removeOne(d->m_importedContexts()[0]);
   }
 }
 
