@@ -49,11 +49,10 @@
 #include "topducontextdata.h"
 #include "duchainregister.h"
 #include "topducontextdynamicdata.h"
-
+//  #define DEBUG_SEARCH
 using namespace KTextEditor;
 
-//Do visibility-caching when more than X items are found.
-const uint visibilityCachingMargin = 10;
+const uint maxApplyAliasesRecursion = 100;
 
 namespace std {
 #if defined(Q_OS_WIN)
@@ -66,26 +65,7 @@ namespace std {
 namespace KDevelop
 {
 
-static Utils::BasicSetRepository* recursiveImportRepository() {
-  static Utils::BasicSetRepository recursiveImportRepository("Recursive Imports", false);
-  return &recursiveImportRepository;
-}
-
-Utils::BasicSetRepository* RecursiveImportRepository::repository() {
-  return recursiveImportRepository();
-}
-
-struct DeclarationTopContextExtractor {
-  inline static IndexedTopDUContext extract(const IndexedDeclaration& decl) {
-    return decl.indexedTopContext();
-  }
-};
-
-struct DUContextTopContextExtractor {
-  inline static IndexedTopDUContext extract(const IndexedDUContext& ctx) {
-    return ctx.indexedTopContext();
-  }
-};
+Utils::BasicSetRepository recursiveImportRepository("Recursive Imports");
 
 ReferencedTopDUContext::ReferencedTopDUContext(TopDUContext* context) : m_topContext(context) {
   if(m_topContext)
@@ -197,7 +177,8 @@ struct TopDUContext::AliasChainElement {
         return current;
     }
     
-    //We do this so we don't create crap items in the qualified-identifier repository while searching
+    //We do this so we don't create crap items in the qualified-identifier repository while searching.
+    //When the qualified identifier doesn't exist, we don't need to search it.
     return QualifiedIdentifier();
   }
 
@@ -733,6 +714,33 @@ const TopDUContext::IndexedRecursiveImports& TopDUContext::recursiveImportIndice
   return m_local->m_indexedRecursiveImports;
 }
 
+void TopDUContextData::updateImportCacheRecursion(uint baseIndex, IndexedTopDUContext currentContext, TopDUContext::IndexedRecursiveImports& visited) {
+  if(visited.contains(currentContext.index()))
+    return;
+  Q_ASSERT(currentContext.index()); //The top-context must be in the repository when this is called
+  if(!currentContext.data()) {
+    kDebug() << "importing invalid context";
+    return;
+  }
+  visited.insert(currentContext.index());
+  
+  const TopDUContextData* currentData = currentContext.data()->topContext()->d_func();
+  if(currentData->m_importsCache.contains(baseIndex) || currentData->m_importsCache.isEmpty())
+  {
+    //If we have a loop or no imports-cache is used, we have to look at each import separately.
+    const KDevelop::DUContext::Import* imports = currentData->m_importedContexts();
+    uint importsSize = currentData->m_importedContextsSize();
+    for(uint a = 0; a < importsSize; ++a) {
+      IndexedTopDUContext next(imports[a].topContextIndex());
+      if(next.isValid())
+        updateImportCacheRecursion(baseIndex, next, visited);
+    }
+  }else{
+    //If we don't have a loop with baseIndex, we can safely just merge with the imported importscache
+    visited += currentData->m_importsCache;
+  }
+}
+
 void TopDUContextData::updateImportCacheRecursion(IndexedTopDUContext currentContext, std::set<uint>& visited) {
   if(visited.find(currentContext.index()) != visited.end())
     return;
@@ -743,20 +751,32 @@ void TopDUContextData::updateImportCacheRecursion(IndexedTopDUContext currentCon
   }
   visited.insert(currentContext.index());
   const TopDUContextData* currentData = currentContext.data()->topContext()->d_func();
-  FOREACH_FUNCTION(const DUContext::Import& import, currentData->m_importedContexts) {
-    IndexedTopDUContext next(import.topContextIndex());
+  const KDevelop::DUContext::Import* imports = currentData->m_importedContexts();
+  uint importsSize = currentData->m_importedContextsSize();
+  for(uint a = 0; a < importsSize; ++a) {
+    IndexedTopDUContext next(imports[a].topContextIndex());
     if(next.isValid())
       updateImportCacheRecursion(next, visited);
   }
 }
 
 
+
 void TopDUContext::updateImportsCache() {
   QMutexLocker lock(&importStructureMutex);
-  std::set<uint> visited;
-  TopDUContextData::updateImportCacheRecursion(this, visited);
-  Q_ASSERT(visited.find(ownIndex()) != visited.end());
-  d_func_dynamic()->m_importsCache = IndexedRecursiveImports(recursiveImportRepository()->createSet(visited));
+  
+  
+  const bool use_fully_recursive_import_cache_computation = false;
+  
+  if(use_fully_recursive_import_cache_computation) {
+    std::set<uint> visited;
+    TopDUContextData::updateImportCacheRecursion(this, visited);
+    Q_ASSERT(visited.find(ownIndex()) != visited.end());
+    d_func_dynamic()->m_importsCache = IndexedRecursiveImports(recursiveImportRepository.createSet(visited));
+  }else{
+    d_func_dynamic()->m_importsCache = IndexedRecursiveImports();
+    TopDUContextData::updateImportCacheRecursion(ownIndex(), this, d_func_dynamic()->m_importsCache);
+  }
   Q_ASSERT(d_func_dynamic()->m_importsCache.contains(IndexedTopDUContext(this)));
   Q_ASSERT(usingImportsCache());
   Q_ASSERT(imports(this, SimpleCursor::invalid()));
@@ -893,6 +913,7 @@ TopDUContext::Features TopDUContext::features() const
 void TopDUContext::setFeatures(Features features)
 {
   features = (TopDUContext::Features)(features & (~((uint)8))); //Remove the "Recursive" flag since that's only for searching
+  features = (TopDUContext::Features)(features & (~ForceUpdateRecursive)); //Remove the update flags
   d_func_dynamic()->m_features = features;
   
   //Replicate features to ParsingEnvironmentFile
@@ -932,20 +953,21 @@ struct TopDUContext::FindDeclarationsAcceptor {
     PersistentSymbolTable::Declarations allDecls;
     
     QualifiedIdentifier id = element.qualifiedIdentifier();
-    if(!id.isEmpty())
-      allDecls = PersistentSymbolTable::self().getDeclarations(id);
+    
+    //If the identifier wasn't found in the identifier repository, the item cannot exist.
+    if(id.isEmpty() && !element.identifier.isEmpty())
+      return;
     
     //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedRecursiveImports, DeclarationTopContextExtractor> filter;
+    PersistentSymbolTable::FilteredDeclarationIterator filter;
     
     //This is used if filterung is disabled
     PersistentSymbolTable::Declarations::Iterator unchecked;
-    if(check.flags & DUContext::NoImportsCheck)
+    if(check.flags & DUContext::NoImportsCheck) {
+      allDecls = PersistentSymbolTable::self().getDeclarations(id);
       unchecked = allDecls.iterator();
-    else
-      filter = ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedRecursiveImports, DeclarationTopContextExtractor>(allDecls.iterator(), top->recursiveImportIndices());
+    } else
+      filter = PersistentSymbolTable::self().getFilteredDeclarations(id, top->recursiveImportIndices());
                                  
     while(filter || unchecked) {
       
@@ -1008,10 +1030,41 @@ bool TopDUContext::findDeclarationsInternal(const SearchItem::PtrList& identifie
   return true;
 }
 
+//This is used to prevent endless recursion due to "using namespace .." declarations, by storing all imports that are already being used.
+struct TopDUContext::ApplyAliasesBuddyInfo {
+  ApplyAliasesBuddyInfo(uint importChainType, ApplyAliasesBuddyInfo* predecessor, IndexedQualifiedIdentifier importId) : m_importChainType(importChainType), m_predecessor(predecessor), m_importId(importId) {
+    if(m_predecessor && m_predecessor->m_importChainType != importChainType)
+      m_predecessor = 0;
+  }
+
+  //May also be called when this is zero.
+  bool alreadyImporting(IndexedQualifiedIdentifier id) {
+    ApplyAliasesBuddyInfo* current = this;
+    while(current) {
+      if(current->m_importId == id)
+        return true;
+      current = current->m_predecessor;
+    }
+    return false;
+  }
+  
+  uint m_importChainType;
+  ApplyAliasesBuddyInfo* m_predecessor;
+  IndexedQualifiedIdentifier m_importId;
+};
+
 ///@todo Implement a cache so at least the global import checks don't need to be done repeatedly. The cache should be thread-local, using DUChainPointer for the hashed items, and when an item was deleted, it should be discarded
 template<class Acceptor>
-void TopDUContext::applyAliases( const AliasChainElement* backPointer, const SearchItem::Ptr& identifier, Acceptor& accept, const SimpleCursor& position, bool canBeNamespace ) const
+void TopDUContext::applyAliases( const AliasChainElement* backPointer, const SearchItem::Ptr& identifier, Acceptor& accept, const SimpleCursor& position, bool canBeNamespace, ApplyAliasesBuddyInfo* buddy, uint recursionDepth ) const
 {
+  if(recursionDepth > maxApplyAliasesRecursion) {
+    QList<QualifiedIdentifier> searches = identifier->toList();
+    QualifiedIdentifier id;
+    if(!searches.isEmpty())
+      id = searches.first();
+    
+    kDebug() << "maximum apply-aliases recursion reached while searching" << id;
+  }
   ///@todo explicitlyGlobal if the first identifier los global
   bool foundAlias = false;
 
@@ -1023,73 +1076,78 @@ void TopDUContext::applyAliases( const AliasChainElement* backPointer, const Sea
 
   if( !identifier->next.isEmpty() || canBeNamespace ) { //If it cannot be a namespace, the last part of the scope will be ignored
 
-    PersistentSymbolTable::Declarations allDecls;
-    
     QualifiedIdentifier id = newElement.qualifiedIdentifier();
-    if(!id.isEmpty())
-      allDecls = PersistentSymbolTable::self().getDeclarations(id);
-    
+
+    if(!id.isEmpty()) {
     //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedRecursiveImports, DeclarationTopContextExtractor> filter(allDecls.iterator(), recursiveImportIndices());
+      PersistentSymbolTable::FilteredDeclarationIterator filter = PersistentSymbolTable::self().getFilteredDeclarations(id, recursiveImportIndices());
 
-    if(filter) {
-      DeclarationChecker check(this, position, AbstractType::Ptr(), NoSearchFlags, 0);
+      if(filter) {
+        DeclarationChecker check(this, position, AbstractType::Ptr(), NoSearchFlags, 0);
 
-      //The first part of the identifier has been found as a namespace-alias.
-      //In c++, we only need the first alias. However, just to be correct, follow them all for now.
-      for(; filter; ++filter)
-      {
-        IndexedDeclaration iDecl(*filter);
-        
-        if(!check(iDecl))
-          continue;
+        //The first part of the identifier has been found as a namespace-alias.
+        //In c++, we only need the first alias. However, just to be correct, follow them all for now.
+        for(; filter; ++filter)
+        {
+          IndexedDeclaration iDecl(*filter);
+          
+          if(!check(iDecl))
+            continue;
 
-        Declaration* aliasDecl = iDecl.data();
-        if(!aliasDecl)
-          continue;
+          Declaration* aliasDecl = iDecl.data();
+          if(!aliasDecl)
+            continue;
 
-        if(aliasDecl->kind() != Declaration::NamespaceAlias)
-          continue;
+          if(aliasDecl->kind() != Declaration::NamespaceAlias)
+            continue;
 
-        if(foundAlias)
-            break;
+          if(foundAlias)
+              break;
 
-        if(aliasDecl->identifier() != newElement.identifier)  //Since we have retrieved the aliases by hash only, we still need to compare the name
-          continue;
+          if(aliasDecl->identifier() != newElement.identifier)  //Since we have retrieved the aliases by hash only, we still need to compare the name
+            continue;
 
-        Q_ASSERT(dynamic_cast<NamespaceAliasDeclaration*>(aliasDecl));
+          Q_ASSERT(dynamic_cast<NamespaceAliasDeclaration*>(aliasDecl));
 
-        NamespaceAliasDeclaration* alias = static_cast<NamespaceAliasDeclaration*>(aliasDecl);
+          NamespaceAliasDeclaration* alias = static_cast<NamespaceAliasDeclaration*>(aliasDecl);
 
-        foundAlias = true;
+          foundAlias = true;
 
-        QualifiedIdentifier importIdentifier = alias->importIdentifier();
+          QualifiedIdentifier importIdentifier = alias->importIdentifier();
 
-        if(importIdentifier.isEmpty()) {
-          kDebug() << "found empty import";
-          continue;
-        }
+          if(importIdentifier.isEmpty()) {
+            kDebug() << "found empty import";
+            continue;
+          }
+          
+          if(buddy->alreadyImporting( importIdentifier ))
+            continue; //This import has already been applied to this search
 
-        //Create a chain of AliasChainElements that represent the identifier
-        uint count = importIdentifier.count();
+          //Create a chain of AliasChainElements that represent the identifier
+          uint count = importIdentifier.count();
 
-        KDevVarLengthArray<AliasChainElement, 5> newChain;
-        newChain.resize(count);
-        for(uint a = 0; a < count; ++a)
-          newChain[a] = AliasChainElement(a == 0 ? 0 : &newChain[a-1], importIdentifier.at(a));
+          KDevVarLengthArray<AliasChainElement, 5> newChain;
+          newChain.resize(count);
+          for(uint a = 0; a < count; ++a)
+            newChain[a] = AliasChainElement(a == 0 ? 0 : &newChain[a-1], importIdentifier.at(a));
 
-        AliasChainElement* newAliasedElement = &newChain[importIdentifier.count()-1];
+          AliasChainElement* newAliasedElement = &newChain[importIdentifier.count()-1];
 
-        if(identifier->next.isEmpty()) {
-          //Just insert the aliased namespace identifier
-          accept(*newAliasedElement);
-        }else{
-          //Create an identifiers where namespace-alias part is replaced with the alias target
-          FOREACH_ARRAY(SearchItem::Ptr item, identifier->next)
-            applyAliases(newAliasedElement, item, accept, position, canBeNamespace);
+          ApplyAliasesBuddyInfo info(1, buddy, importIdentifier);
+          
+          if(identifier->next.isEmpty()) {
+            //Just insert the aliased namespace identifier
+            accept(*newAliasedElement);
+          }else{
+            //Create an identifiers where namespace-alias part is replaced with the alias target
+            FOREACH_ARRAY(SearchItem::Ptr item, identifier->next)
+              applyAliases(newAliasedElement, item, accept, position, canBeNamespace, &info, recursionDepth+1);
+          }
         }
       }
+    }else{
+      if(!identifier->identifier.isEmpty())
+        return; //Normally the id should not be empty, but it is, so it wasn't found in the repo. Nothing to do.
     }
   }
 
@@ -1098,7 +1156,7 @@ void TopDUContext::applyAliases( const AliasChainElement* backPointer, const Sea
       accept(newElement); //We're at the end of a qualified identifier, accept it
     } else {
       FOREACH_ARRAY(SearchItem::Ptr next, identifier->next)
-        applyAliases(&newElement, next, accept, position, canBeNamespace);
+        applyAliases(&newElement, next, accept, position, canBeNamespace, 0, recursionDepth+1);
     }
   }
 
@@ -1115,61 +1173,60 @@ void TopDUContext::applyAliases( const AliasChainElement* backPointer, const Sea
 
     QualifiedIdentifier id = importChainItem.qualifiedIdentifier();
     
-    PersistentSymbolTable::Declarations allDecls;
-    if(!id.isEmpty())
-      allDecls = PersistentSymbolTable::self().getDeclarations(id);
-    
-    //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetTreeFilterIterator<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, 
-                                 IndexedRecursiveImports, DeclarationTopContextExtractor> filter(allDecls.iterator(), recursiveImportIndices());
-    
+    if(!id.isEmpty()) {
+      //This iterator efficiently filters the visible declarations out of all declarations
+      PersistentSymbolTable::FilteredDeclarationIterator filter = PersistentSymbolTable::self().getFilteredDeclarations(id, recursiveImportIndices());
       
-    if(filter) {
-      DeclarationChecker check(this, position, AbstractType::Ptr(), NoSearchFlags, 0);
+        
+      if(filter) {
+        DeclarationChecker check(this, position, AbstractType::Ptr(), NoSearchFlags, 0);
 
-      for(; filter; ++filter)
-      {
-        //We must never break or return from this loop, because else we might be creating a bad cache
-        if(!check(*filter))
-          continue;
+        for(; filter; ++filter)
+        {
+          //We must never break or return from this loop, because else we might be creating a bad cache
+          if(!check(*filter))
+            continue;
 
-        Declaration* importDecl = filter->data();
-        if(!importDecl)
-          continue;
+          Declaration* importDecl = filter->data();
+          if(!importDecl)
+            continue;
 
-        if(importDecl->identifier() != globalImportIdentifier) //We need to check, since we've only searched by hash
-          continue;
+          //Search for the identifier with the import-identifier prepended
+          Q_ASSERT(dynamic_cast<NamespaceAliasDeclaration*>(importDecl));
+          NamespaceAliasDeclaration* alias = static_cast<NamespaceAliasDeclaration*>(importDecl);
 
-        //Search for the identifier with the import-identifier prepended
-        Q_ASSERT(dynamic_cast<NamespaceAliasDeclaration*>(importDecl));
-        NamespaceAliasDeclaration* alias = static_cast<NamespaceAliasDeclaration*>(importDecl);
+  #ifdef DEBUG_SEARCH
+          kDebug() << "found import of" << alias->importIdentifier().toString();
+  #endif
 
-#ifdef DEBUG_SEARCH
-        kDebug() << "found import of" << alias->importIdentifier().toString();
-#endif
+          QualifiedIdentifier importIdentifier = alias->importIdentifier();
 
-        QualifiedIdentifier importIdentifier = alias->importIdentifier();
+          if(importIdentifier.isEmpty()) {
+            kDebug() << "found empty import";
+            continue;
+          }
 
-        if(importIdentifier.isEmpty()) {
-          kDebug() << "found empty import";
-          continue;
+          if(buddy->alreadyImporting( importIdentifier ))
+            continue; //This import has already been applied to this search
+
+          ApplyAliasesBuddyInfo info(2, buddy, importIdentifier);
+
+          int count = importIdentifier.count();
+          KDevVarLengthArray<AliasChainElement, 5> newChain;
+          newChain.resize(importIdentifier.count());
+
+          for(int a = 0; a < count; ++a)
+            newChain[a] = AliasChainElement(a == 0 ? 0 : &newChain[a-1], importIdentifier.at(a));
+
+          AliasChainElement* newAliasedElement = &newChain[count-1];
+
+  #ifdef DEBUG_SEARCH
+          kDebug() << "imported" << newAliasedElement->qualifiedIdentifier().toString();
+  #endif
+          //Prevent endless recursion by checking whether we're actually doing a change
+          if(!backPointer || newAliasedElement->hash != backPointer->hash || newAliasedElement->qualifiedIdentifier() != backPointer->qualifiedIdentifier())
+          applyAliases(newAliasedElement, identifier, accept, importDecl->topContext() == this ? importDecl->range().start : position, canBeNamespace, &info, recursionDepth+1);
         }
-
-        int count = importIdentifier.count();
-        KDevVarLengthArray<AliasChainElement, 5> newChain;
-        newChain.resize(importIdentifier.count());
-
-        for(int a = 0; a < count; ++a)
-          newChain[a] = AliasChainElement(a == 0 ? 0 : &newChain[a-1], importIdentifier.at(a));
-
-        AliasChainElement* newAliasedElement = &newChain[count-1];
-
-#ifdef DEBUG_SEARCH
-        kDebug() << "imported" << newAliasedElement->qualifiedIdentifier().toString();
-#endif
-        //Prevent endless recursion by checking whether we're actually doing a change
-        if(!backPointer || newAliasedElement->hash != backPointer->hash || newAliasedElement->qualifiedIdentifier() != backPointer->qualifiedIdentifier())
-        applyAliases(newAliasedElement, identifier, accept, position, canBeNamespace);
       }
     }
   }
@@ -1179,7 +1236,7 @@ template<class Acceptor>
 void TopDUContext::applyAliases( const SearchItem::PtrList& identifiers, Acceptor& acceptor, const SimpleCursor& position, bool canBeNamespace ) const
 {
   FOREACH_ARRAY(SearchItem::Ptr item, identifiers)
-    applyAliases(0, item, acceptor, position, canBeNamespace);
+    applyAliases(0, item, acceptor, position, canBeNamespace, 0, 0);
 }
 
 struct TopDUContext::FindContextsAcceptor {
@@ -1194,19 +1251,21 @@ struct TopDUContext::FindContextsAcceptor {
     PersistentSymbolTable::Contexts allDecls;
     
     QualifiedIdentifier id = element.qualifiedIdentifier();
-    allDecls = PersistentSymbolTable::self().getContexts(id);
+    
+    //If the identifier wasn't found in the identifier repository, the item cannot exist.
+    if(id.isEmpty() && !element.identifier.isEmpty())
+      return;
     
     //This iterator efficiently filters the visible declarations out of all declarations
-    ConvenientEmbeddedSetTreeFilterIterator<IndexedDUContext, IndexedDUContextHandler, IndexedTopDUContext, 
-                                 IndexedRecursiveImports, DUContextTopContextExtractor> filter;
+    PersistentSymbolTable::FilteredDUContextIterator filter;
     
     //This is used if filterung is disabled
     PersistentSymbolTable::Contexts::Iterator unchecked;
-    if(check.flags & DUContext::NoImportsCheck)
+    if(check.flags & DUContext::NoImportsCheck) {
+      allDecls = PersistentSymbolTable::self().getContexts(id);
       unchecked = allDecls.iterator();
-    else
-      filter = ConvenientEmbeddedSetTreeFilterIterator<IndexedDUContext, IndexedDUContextHandler, IndexedTopDUContext, 
-                                 IndexedRecursiveImports, DUContextTopContextExtractor>(allDecls.iterator(), top->recursiveImportIndices());
+    } else
+      filter = PersistentSymbolTable::self().getFilteredContexts(id, top->recursiveImportIndices());
 
     while(filter || unchecked) {
       
@@ -1432,7 +1491,7 @@ int TopDUContext::indexForUsedDeclaration(Declaration* declaration, bool create)
 if(!declaration)
   return std::numeric_limits<int>::max();
   
-  if(declaration->topContext() == this) {
+  if(declaration->topContext() == this && !declaration->inSymbolTable()) {
     uint index = declaration->ownIndex();
     Q_ASSERT(!(index & (1<<31)));
     return (int)(index | (1<<31)); //We don't put context-local declarations into the list, that's a waste. We just use the mark them with the highest bit.
