@@ -41,8 +41,12 @@ namespace KDevelop
   class DeclarationChecker;
   class TopDUContext;
 
+  KDEVPLATFORMLANGUAGE_EXPORT extern Utils::BasicSetRepository recursiveImportRepository;
+  
   struct KDEVPLATFORMLANGUAGE_EXPORT RecursiveImportRepository {
-    static Utils::BasicSetRepository* repository();
+    inline static Utils::BasicSetRepository* repository() {
+      return &recursiveImportRepository;
+    }
   };
   
   ///Maps an imported top-context to a pair:
@@ -211,7 +215,9 @@ public:
     ///When this flag is set, also _all_ recursive imports have to be computed with AllDeclarationsContextsAndUses
     ///This flag can not be set on a context, it is only used as a parameter to several updating functions. When you set it
     ///on a top-context, its flag will be AllDeclarationsContextsAndUses.
-    AllDeclarationsContextsAndUsesForRecursive = 8 + AllDeclarationsContextsAndUses
+    AllDeclarationsContextsAndUsesForRecursive = 8 + AllDeclarationsContextsAndUses,
+    ForceUpdate = 16, //This flag can not be set on a context, but is only used during updating
+    ForceUpdateRecursive = ForceUpdate | 32  //This flag can not be set on a context, but is only used during updating
   };
   
   Features features() const;
@@ -351,9 +357,10 @@ protected:
   struct FindContextsAcceptor;
   struct DeclarationChecker;
   struct ContextChecker;
+  struct ApplyAliasesBuddyInfo;
 
   template<class Acceptor>
-  void applyAliases( const AliasChainElement* backPointer, const SearchItem::Ptr& identifier, Acceptor& acceptor, const SimpleCursor& position, bool canBeNamespace ) const;
+  void applyAliases( const AliasChainElement* backPointer, const SearchItem::Ptr& identifier, Acceptor& acceptor, const SimpleCursor& position, bool canBeNamespace, ApplyAliasesBuddyInfo* buddy, uint recursionDepth ) const;
   //Same as imports, without the slow access-check, for internal usage
   bool importsPrivate(const DUContext * origin, const SimpleCursor& position) const;
   DUCHAIN_DECLARE_DATA(TopDUContext)
