@@ -48,25 +48,23 @@ ProjectBuildSetWidget::ProjectBuildSetWidget( QWidget* parent )
      m_ui( new Ui::ProjectBuildSetWidget )
 {
     m_ui->setupUi( this );
-    m_ui->addItemButton->setToolButtonStyle( Qt::ToolButtonIconOnly );
+    
     m_ui->addItemButton->setIcon( KIcon( "list-add" ) );
-
     connect( m_ui->addItemButton, SIGNAL( clicked() ),
              this, SLOT( addItems() ) );
 
-    m_ui->removeItemButton->setToolButtonStyle( Qt::ToolButtonIconOnly );
     m_ui->removeItemButton->setIcon( KIcon( "list-remove" ) );
-
     connect( m_ui->removeItemButton, SIGNAL( clicked() ),
              this, SLOT( removeItems() ) );
 
+    m_ui->buildButton->setIcon( KIcon( "run-build" ) );
+    m_ui->installButton->setIcon( KIcon( "run-install" ) );
+    m_ui->cleanButton->setIcon( KIcon( "run-clean" ) );
 
     m_ui->itemView->horizontalHeader()->setStretchLastSection(true);
     m_ui->itemView->setContextMenuPolicy( Qt::CustomContextMenu );
     connect( m_ui->itemView, SIGNAL( customContextMenuRequested( const QPoint& ) ),
              SLOT(showContextMenu(const QPoint&) ) );
-    connect( m_ui->itemView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&) ),
-             this, SLOT(selectionChanged()) );
     layout()->setMargin(0);
 }
 
@@ -74,10 +72,20 @@ void ProjectBuildSetWidget::setProjectView( ProjectManagerView* view )
 {
     m_view = view;
     m_ui->itemView->setModel( m_view->plugin()->buildSet() );
+    connect( m_ui->itemView->selectionModel(), SIGNAL(selectionChanged(const QItemSelection&, const QItemSelection&) ),
+             this, SLOT(selectionChanged()) );
+
+    connect( m_ui->buildButton, SIGNAL( clicked() ),
+             m_view->plugin(), SLOT( buildProjectItems() ) );
+    connect( m_ui->installButton, SIGNAL( clicked() ),
+             m_view->plugin(), SLOT( installProjectItems() ) );
+    connect( m_ui->cleanButton, SIGNAL( clicked() ),
+             m_view->plugin(), SLOT( cleanProjectItems() ) );
 }
 
 void ProjectBuildSetWidget::selectionChanged()
 {
+    kDebug() << "checking selectionmodel:" << m_ui->itemView->selectionModel()->selectedRows();
     m_ui->removeItemButton->setEnabled( !m_ui->itemView->selectionModel()->selectedRows().isEmpty() );
     m_ui->addItemButton->setEnabled( !m_view->selectedItems().isEmpty() );
 }
@@ -106,9 +114,8 @@ void ProjectBuildSetWidget::addItems()
 void ProjectBuildSetWidget::removeItems()
 {
     QList<int> rows;
-    foreach( QItemSelectionRange range, m_ui->itemView->selectionModel()->selection() )
+    foreach( const QItemSelectionRange &range, m_ui->itemView->selectionModel()->selection() )
     {
         m_view->plugin()->buildSet()->removeRows( range.top(), range.height() );
     }
 }
-
