@@ -210,7 +210,19 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
         }
         
-        if(!hasTargets && item->executable())
+        if ( item->file() )
+        {
+            KAction* action = new KAction( i18n( "Remove File" ), this );
+            connect( action, SIGNAL(triggered()), this, SLOT(removeFileFromContextMenu()) );
+            menuExt.addAction( ContextMenuExtension::FileGroup, action );
+        }
+        else if ( item->target() )
+        {
+            KAction* action = new KAction( i18n( "Create File" ), this );
+            connect( action, SIGNAL(triggered()), this, SLOT(createFileInTargetFromContextMenu()) );
+            menuExt.addAction( ContextMenuExtension::FileGroup, action );
+        }
+        else if(!hasTargets && item->executable())
         {
             KAction* action = new KAction( i18n("Run"), this );
             connect( action, SIGNAL( triggered() ), this, SLOT( runTargetsFromContextMenu() ) );
@@ -386,10 +398,10 @@ void ProjectManagerViewPlugin::removeFolderFromContextMenu()
         if ( item->folder() ) {
             QWidget* window(ICore::self()->uiController()->activeMainWindow()->window());
             int q=KMessageBox::questionYesNo(window, i18n("Do you want to remove the directory from the filesystem too?"));
-            if(q)
+            if(q==KMessageBox::Yes)
             {
                 if ( !KIO::NetAccess::del( item->folder()->url(), window ) ) {
-                    KMessageBox::error( window, i18n( "Can't remove folder." ) );
+                    KMessageBox::error( window, i18n( "Cannot remove folder." ) );
                     continue;
                 }
             }
@@ -399,39 +411,81 @@ void ProjectManagerViewPlugin::removeFolderFromContextMenu()
     }
 }
 
+void ProjectManagerViewPlugin::removeFileFromContextMenu()
+{
+    foreach( KDevelop::ProjectBaseItem* item, d->ctxProjectItemList )
+    {
+        if ( item->file() ) {
+            QWidget* window(ICore::self()->uiController()->activeMainWindow()->window());
+            int q=KMessageBox::questionYesNo(window, i18n("Do you want to remove the file from the filesystem too?"));
+            if(q==KMessageBox::Yes)
+            {
+                if ( !KIO::NetAccess::del( item->file()->url(), window ) ) {
+                    KMessageBox::error( window, i18n( "Cannot remove the file." ) );
+                    continue;
+                }
+            }
+            
+            item->project()->projectFileManager()->removeFile(item->file());
+        }
+    }
+}
+
+ProjectFileItem* createFile(const ProjectFolderItem* item)
+{
+    QWidget* window = ICore::self()->uiController()->activeMainWindow()->window();
+    QString name = QInputDialog::getText(window, i18n("Create File"), i18n("File Name"));
+    
+    if(name.isEmpty())
+        return 0;
+    
+    KUrl url=item->url();
+    url.addPath( name );
+
+    if (KIO::NetAccess::exists( url, KIO::NetAccess::SourceSide, window )) {
+        KMessageBox::error( window, i18n( "This file exists already." ) );
+        return 0;
+    }
+
+    KSaveFile file(url.path());
+    if ( ! file.open() ) {
+        KMessageBox::error( window, i18n( "Can't create file." ) );
+        return 0;
+    }
+    file.finalize();
+    file.close();
+
+    ProjectFileItem* ret=item->project()->projectFileManager()->addFile( url, item->folder() );
+    ICore::self()->documentController()->openDocument( url );
+    return ret;
+}
+
 void ProjectManagerViewPlugin::createFileFromContextMenu( )
 {
     foreach( KDevelop::ProjectBaseItem* item, d->ctxProjectItemList )
     {
         if ( item->folder() ) {
-            QWidget* window(ICore::self()->uiController()->activeMainWindow()->window());
-            QString name = QInputDialog::getText ( window,
-                                i18n ( "Create File" ), i18n ( "File Name" ) );
-            if (!name.isEmpty()) {
-                KUrl url = item->folder()->url();
-                url.addPath( name );
+            createFile(item->folder());
+        }
+    }
+}
 
-                if (KIO::NetAccess::exists( url, KIO::NetAccess::SourceSide, window )) {
-                    KMessageBox::error( window, i18n( "This file exists already." ) );
-                    continue;
-                }
-
-                KSaveFile file(url.path());
-                if ( ! file.open() ) {
-                    KMessageBox::error( window, i18n( "Can't create file." ) );
-                    continue;
-                }
-                file.finalize();
-                file.close();
-
-                item->project()->projectFileManager()->addFile( url, item->folder() );
-                ICore::self()->documentController()->openDocument( url );
+void ProjectManagerViewPlugin::createFileInTargetFromContextMenu( )
+{
+    foreach( KDevelop::ProjectBaseItem* item, d->ctxProjectItemList )
+    {
+        if ( item->target() )
+        {
+            ProjectFolderItem* folder=dynamic_cast<ProjectFolderItem*>(item->parent());
+            if(folder)
+            {
+                ProjectFileItem* f=createFile(folder);
+                if(f)
+                    item->project()->buildSystemManager()->addFileToTarget(f, item->target());
             }
         }
     }
 }
 
-
-
 #include "projectmanagerviewplugin.moc"
 
