@@ -75,7 +75,9 @@ protected Q_SLOTS:
     void reloadFromContextMenu();
     void createFolderFromContextMenu();
     void createFileFromContextMenu();
+    void createFileInTargetFromContextMenu();
     void removeFolderFromContextMenu();
+    void removeFileFromContextMenu();
 
 private:
     static KDevelop::IProjectBuilder* getProjectBuilder( KDevelop::ProjectBaseItem* item );
