@@ -46,12 +46,11 @@ QString DUChainItemData::text() const {
       if(def->declaration())
           decl = def->declaration();
       
-  QString text;
+  QString text = decl->qualifiedIdentifier().toString();
+  
   TypePtr<FunctionType> function = decl->type<FunctionType>();
   if( function )
-    text  = QString("%1%2").arg(decl->qualifiedIdentifier().toString()).arg(function->partToString( FunctionType::SignatureArguments ));
-  else
-    text = m_item.m_text;
+    text  += function->partToString( FunctionType::SignatureArguments );
 
   return text;
 }
@@ -66,17 +65,16 @@ QList<QVariant> DUChainItemData::highlighting() const {
       if(def->declaration())
           decl = def->declaration();
 
-  TypePtr<FunctionType> function = decl->type<FunctionType>();
-  if(!function)
-    return QList<QVariant>();
-
   QTextCharFormat boldFormat;
   boldFormat.setFontWeight(QFont::Bold);
   QTextCharFormat normalFormat;
 
   int prefixLength = 0;
 
-  QString signature = function->partToString( FunctionType::SignatureArguments );
+  QString signature;
+  TypePtr<FunctionType> function = decl->type<FunctionType>();
+  if(function)
+    signature = function->partToString( FunctionType::SignatureArguments );
 
   //Only highlight the last part of the qualified identifier, so the scope doesn't distract too much
   QualifiedIdentifier id = decl->qualifiedIdentifier();
@@ -94,9 +92,11 @@ QList<QVariant> DUChainItemData::highlighting() const {
   ret << prefixLength;
   ret << lastId.length();
   ret << QVariant(boldFormat);
-  ret << prefixLength + lastId.length();
-  ret << signature.length();
-  ret << QVariant(normalFormat);
+  if(!signature.isEmpty()) {
+    ret << prefixLength + lastId.length();
+    ret << signature.length();
+    ret << QVariant(normalFormat);
+  }
 
   return ret;
 
@@ -115,9 +115,9 @@ QString DUChainItemData::htmlDescription() const {
   QString text;
 
   if( function && function->returnType() )
-    text  = i18n("Return:") + " " + function->partToString(FunctionType::SignatureReturn);
+    text  = i18n("Return:") + ' ' + function->partToString(FunctionType::SignatureReturn);
 
-  text += " " + i18n("File:") + " " + decl->url().str();
+  text += ' ' + i18n("File:") + ' ' + decl->url().str();
 //   TypePtr<FunctionType> function = decl->type<FunctionType>();
 //   if( function )
 //     text  = QString("%1 %2%3").arg(function->partToString( FunctionType::SignatureReturn)).arg(decl->identifier().toString()).arg(function->partToString( FunctionType::SignatureArguments ));
@@ -126,7 +126,7 @@ QString DUChainItemData::htmlDescription() const {
 
   QString ret = "<small><small>" + text;
   if(!m_item.m_project.isEmpty()) {
-      ret = i18n("Project") + " " + m_item.m_project + (ret.isEmpty() ? ", " : "") + ret/*", " + i18n("path") + totalUrl().path() +*/; //Show only the path because of limited space
+      ret = i18n("Project") + ' ' + m_item.m_project + (ret.isEmpty() ? ", " : "") + ret/*", " + i18n("path") + totalUrl().path() +*/; //Show only the path because of limited space
   }
 
   ret += "</small></small>";
@@ -169,7 +169,7 @@ QWidget* DUChainItemData::expandingWidget() const {
   if( !decl || !decl->context() )
     return 0;
 
-  return decl->context()->createNavigationWidget( decl, decl->topContext(), m_item.m_project.isEmpty() ? QString() : ("<small><small>" + i18n("Project") + " " + m_item.m_project + "<br>" + "</small></small>"));
+  return decl->context()->createNavigationWidget( decl, decl->topContext(), m_item.m_project.isEmpty() ? QString() : ("<small><small>" + i18n("Project") + ' ' + m_item.m_project + "<br>" + "</small></small>"));
 }
 
 QIcon DUChainItemData::icon() const {
