@@ -72,6 +72,7 @@ bool hasMandatoryProperties( const KPluginInfo& info )
            && version.isValid() && version.canConvert( QVariant::String );
 }
 
+
 class PluginControllerPrivate
 {
 public:
@@ -93,6 +94,39 @@ public:
     };
     CleanupMode cleanupMode;
 
+    bool canUnload( const KPluginInfo& plugin )
+    {
+        QStringList interfaces=plugin.property( "X-KDevelop-Interfaces" ).toStringList();
+        foreach( const KPluginInfo& info, loadedPlugins.keys() )
+        {
+            if( info.pluginName() != plugin.pluginName() ) 
+            {
+                QStringList dependencies = info.property( "X-KDevelop-IRequired" ).toStringList();
+                dependencies += info.property( "X-KDevelop-IOptional" ).toStringList();
+                foreach( const QString& dep, dependencies )
+                {
+                    if( interfaces.contains( dep ) && !canUnload( info ) )
+                    {
+                        return false;
+                    }
+                }
+            }
+        }
+        return true;
+    }
+
+    KPluginInfo infoForId( const QString& id ) const
+    {
+        foreach( const KPluginInfo& info, plugins )
+        {
+            if( info.pluginName() == id )
+            {
+                return info;
+            }
+        }
+        return KPluginInfo();
+    }
+
     Core *core;
     QExtensionManager* m_manager;
 };
@@ -164,25 +198,33 @@ IPlugin* PluginController::loadPlugin( const QString& pluginName )
 
 void PluginController::initialize()
 {
-    KConfigGroup grp = Core::self()->activeSession()->config()->group( pluginControllerGrp );
-
-    QMap<QString, QString> entries = grp.entryMap();
-
     QMap<QString, bool> pluginMap;
+    // Get the default from the ShellExtension
+    foreach( const QString& s, ShellExtension::getInstance()->defaultPlugins() )
+    {
+        pluginMap.insert( s, true );
+    }
+
+    KConfigGroup grp = Core::self()->activeSession()->config()->group( pluginControllerGrp );
+    QMap<QString, QString> entries = grp.entryMap();
 
     QMap<QString, QString>::Iterator it;
     for ( it = entries.begin(); it != entries.end(); ++it )
     {
         QString key = it.key();
-        if ( key.endsWith( QLatin1String( "Enabled" ) ) )
-            pluginMap.insert( key.left(key.length() - 7), grp.readEntry(key,false) );
-    }
-
-    // If pluginMap is empty here (i.e. nothing in the session) use the list available
-    // from the shellextension
-    foreach( const QString& s, ShellExtension::getInstance()->defaultPlugins() )
-    {
-        pluginMap.insert( s, true );
+        if ( key.endsWith( QLatin1String( "Enabled" ) ) ) 
+        {
+            QString pluginid = key.left( key.length() - 7 );
+            bool defValue;
+            QMap<QString, bool>::const_iterator entry = pluginMap.constFind( pluginid );
+            if( entry != pluginMap.constEnd() )
+            {
+                defValue = entry.value();
+            } else {
+                defValue = false;
+            }
+            pluginMap.insert( key.left(key.length() - 7), grp.readEntry(key,defValue) );
+        }
     }
 
     foreach( const KPluginInfo& pi, d->plugins )
@@ -190,10 +232,13 @@ void PluginController::initialize()
         if( isGlobalPlugin( pi ) )
         {
             QMap<QString, bool>::const_iterator it = pluginMap.constFind( pi.pluginName() );
-            if( pluginMap.isEmpty() || ( it != pluginMap.constEnd() && it.value() ) && loadPluginInternal( pi.pluginName() ) != 0 )
-            {
+            if( pluginMap.isEmpty() || ( it != pluginMap.constEnd() && it.value() ) )
+            { 
                 loadPluginInternal( pi.pluginName() );
-                grp.writeEntry( pi.pluginName()+"Enabled", true );
+                if( pluginMap.isEmpty() || it == pluginMap.constEnd() )
+                {
+                    grp.writeEntry( pi.pluginName()+"Enabled", true );
+                }
             }
         }
     }
@@ -206,12 +251,16 @@ QList<IPlugin *> PluginController::loadedPlugins() const
     return d->loadedPlugins.values();
 }
 
-void PluginController::unloadPlugin( const QString & pluginId )
+bool PluginController::unloadPlugin( const QString & pluginId )
 {
-    if( IPlugin *thePlugin = plugin( pluginId ) )
+    IPlugin *thePlugin = plugin( pluginId );
+    bool canUnload = d->canUnload( d->infoForId( pluginId ) );
+    if( thePlugin && canUnload )
     {
         unloadPlugin(thePlugin, Later);
+        return true;
     }
+    return (canUnload && thePlugin);
 }
 
 void PluginController::unloadPlugin(IPlugin* plugin, PluginDeletion deletion)
@@ -520,6 +569,26 @@ QList<KPluginInfo> PluginController::allPluginInfos() const
 
 void PluginController::updateLoadedPlugins()
 {
+    QStringList defaultPlugins = ShellExtension::getInstance()->defaultPlugins();
+    KConfigGroup grp = Core::self()->activeSession()->config()->group( pluginControllerGrp );
+    foreach( const KPluginInfo& info, d->plugins )
+    {
+        if( isGlobalPlugin( info ) )
+        {
+            bool enabled = grp.readEntry( info.pluginName()+"Enabled", ( defaultPlugins.isEmpty() || defaultPlugins.contains( info.pluginName() ) ) );
+            if( d->loadedPlugins.contains( info ) && !enabled ) 
+            {
+                kDebug() << "unloading" << info.pluginName();
+                if( !unloadPlugin( info.pluginName() ) ) 
+                {
+                    grp.writeEntry( info.pluginName()+"Enabled", false );
+                }
+            } else if( !d->loadedPlugins.contains( info ) && enabled ) 
+            {
+                loadPluginInternal( info.pluginName() );
+            }
+        }
+    }
 }
 
 void PluginController::resetToDefaults()
