@@ -37,14 +37,20 @@ Boston, MA 02110-1301, USA.
 #include <kxmlguiwindow.h>
 #include <kfiledialog.h>
 #include <kactioncollection.h>
+#include <ktemporaryfile.h>
 #include <kservicetypetrader.h>
 #include <krecentfilesaction.h>
 #include <kactionmenu.h>
+#include <ksharedconfig.h>
+#include <kconfiggroup.h>
 #include <ksettings/dialog.h>
+#include <kio/netaccess.h>
 #include <kstandarddirs.h>
+#include <kpassivepopup.h>
 
 #include "sublime/area.h"
 #include <interfaces/iplugin.h>
+#include <interfaces/isession.h>
 #include <project/interfaces/iprojectfilemanager.h>
 #include <project/projectmodel.h>
 #include <interfaces/ilanguagecontroller.h>
@@ -57,6 +63,7 @@ Boston, MA 02110-1301, USA.
 #include "plugincontroller.h"
 #include "uicontroller.h"
 #include "documentcontroller.h"
+#include "openprojectdialog.h"
 
 namespace KDevelop
 {
@@ -120,7 +127,7 @@ public:
         QList<IPlugin*> plugins = m_core->pluginController()->loadedPlugins();
         QStringList pluginnames;
         kDebug() << "managerplugin:" << project->managerPlugin();
-        for( QList<IPlugin*>::iterator it = plugins.begin(); it != plugins.end(); it++ )
+        for( QList<IPlugin*>::iterator it = plugins.begin(); it != plugins.end(); ++it )
         {
             IPlugin* plugin = *it;
             IProjectFileManager* iface = plugin->extension<KDevelop::IProjectFileManager>();
@@ -145,22 +152,49 @@ ProjectDialogProvider::ProjectDialogProvider(ProjectControllerPrivate* const p)
 ProjectDialogProvider::~ProjectDialogProvider()
 {}
 
+void writeNewProjectFile( KSharedConfig::Ptr cfg, const QString& name, const QString& manager )
+{
+    KConfigGroup grp = cfg->group( "Project" );
+    grp.writeEntry( "Name", name );
+    grp.writeEntry( "Manager", manager );
+    cfg->sync();
+}
+
 KUrl ProjectDialogProvider::askProjectConfigLocation()
 {
     Q_ASSERT(d);
-    KSharedConfig * config = KGlobal::config().data();
-    KConfigGroup group = config->group( "General Options" );
-    QString dir = group.readEntry( "DefaultProjectsDirectory",
-                                     QDir::homePath() );
-
-    QString projectFileInfo = ShellExtension::getInstance()->projectFileExtension()
-        + "|" + ShellExtension::getInstance()->projectFileDescription()
-        + "\n";
-    KUrl response = KFileDialog::getOpenUrl( dir, projectFileInfo,
-        d->m_core->uiControllerInternal()->defaultMainWindow(),
-        i18n( "Open Project" ) );
-    return response;
+    OpenProjectDialog dlg( Core::self()->uiController()->activeMainWindow() );
+    if(dlg.exec() == QDialog::Rejected)
+        return KUrl();
+    
+    KUrl dir = dlg.directory();
+    QString file = dlg.projectFile();
+    kDebug() << "selected project:" << dir << file << dlg.projectName() << dlg.projectManager();
+    if( file.isEmpty() )
+    {
+        dir.addPath( dir.fileName() + "." + ShellExtension::getInstance()->projectFileExtension() );
+        if( dir.isLocalFile() )
+        {
+            writeNewProjectFile( KSharedConfig::openConfig( dir.toLocalFile(), KConfig::SimpleConfig ),
+                            dlg.projectName(),
+                            dlg.projectManager() );
+        } else
+        {
+            KTemporaryFile tmp;
+            tmp.setAutoRemove( false );
+            tmp.open();
+            tmp.close();
+            writeNewProjectFile( KSharedConfig::openConfig( tmp.fileName(), KConfig::SimpleConfig ),
+                            dlg.projectName(),
+                            dlg.projectManager() );
+            KIO::NetAccess::upload( tmp.fileName(), dir, Core::self()->uiControllerInternal()->defaultMainWindow() );
+        }
+    } else 
+    {
+        dir.addPath( file );
     }
+    return dir;
+}
 
 bool ProjectDialogProvider::userWantsReopen()
 {
@@ -244,7 +278,7 @@ ProjectController::~ProjectController()
 
 void ProjectController::cleanup()
 {
-    KSharedConfig * config = KGlobal::config().data();
+    KSharedConfig::Ptr config = Core::self()->activeSession()->config();
     KConfigGroup group = config->group( "General Options" );
 
     KUrl::List openProjects;
@@ -260,7 +294,7 @@ void ProjectController::cleanup()
 void ProjectController::initialize()
 {
     if (d->reopenProjectsOnStartup) {
-        KSharedConfig * config = KGlobal::config().data();
+        KSharedConfig::Ptr config = Core::self()->activeSession()->config();
         KConfigGroup group = config->group( "General Options" );
         KUrl::List openProjects = group.readEntry( "Open Projects", QStringList() );
 
@@ -334,6 +368,8 @@ bool ProjectController::openProject( const KUrl &projectFile )
         }
     }
 
+    KPassivePopup::message( i18n( "Loading Project: %1", url.prettyUrl() ), Core::self()->uiControllerInternal()->activeMainWindow() );
+
     //FIXME Create the hidden directory if it doesn't exist
     if ( loadProjectPart() )
     {
@@ -404,6 +440,7 @@ bool ProjectController::projectImportingFinished( IProject* project )
         }
         Core::self()->languageController()->backgroundParser()->addDocumentList( parseList, KDevelop::TopDUContext::AllDeclarationsContextsAndUses, 10 );
     }
+    KPassivePopup::message( i18n( "Project loaded: %1", project->name() ), Core::self()->uiControllerInternal()->activeMainWindow() );
 
     return true;
 }
