@@ -32,6 +32,8 @@ Boston, MA 02110-1301, USA.
 #include <KDebug>
 #include <KColorScheme>
 #include <KCompositeJob>
+#include <kdialogjobuidelegate.h>
+
 #include <interfaces/iproject.h>
 #include <interfaces/idocumentcontroller.h>
 #include <outputview/ioutputview.h>
@@ -79,8 +81,12 @@ RunController::RunController(QObject *parent)
 class ExecuteCompositeJob : public KCompositeJob
 {
     public:
-        ExecuteCompositeJob(QObject* parent, const QList<KJob*>& jobs) : KCompositeJob(parent)
+        ExecuteCompositeJob(QObject* parent, const QList<KJob*>& jobs)
+            : KCompositeJob(parent)
+            , m_killing(false)
         {
+            setCapabilities(Killable);
+
             qDebug() << "execute composite" << jobs;
             foreach(KJob* job, jobs) {
                 addSubjob(job);
@@ -101,7 +107,7 @@ class ExecuteCompositeJob : public KCompositeJob
             kDebug() << "finished: "<< job << job->error() << error();
             KCompositeJob::slotResult(job);
 
-            if(hasSubjobs() && !error())
+            if(hasSubjobs() && !error() && !m_killing)
             {
                 kDebug() << "remaining: " << subjobs().count() << subjobs();
                 KJob* nextJob=subjobs().first();
@@ -110,6 +116,25 @@ class ExecuteCompositeJob : public KCompositeJob
                 emitResult();
             }
         }
+
+    protected:
+        virtual bool doKill()
+        {
+            m_killing = true;
+            if(hasSubjobs()) {
+                KJob* j = subjobs().first();
+                if (j->kill()) {
+                    removeSubjob(j);
+                    return true;
+                } else {
+                    return false;
+                }
+            }
+            return true;
+        }
+
+    private:
+        bool m_killing;
 };
 
 KJob* RunController::execute(const IRun & run)
@@ -328,7 +353,7 @@ IRun KDevelop::RunController::defaultRun() const
             {
                 KMessageBox::error(0, i18n("Target '%1' is not executable.", target));
             }
-        } else 
+        } else
         {
             KMessageBox::error(0, i18n("Target '%1' could not be found.", target));
         }
@@ -338,13 +363,10 @@ IRun KDevelop::RunController::defaultRun() const
     run.setWorkingDirectory(group.readEntry("Working Directory", QString()));
     run.setArguments(splitArguments(group.readEntry("Arguments", QString())));
     if (group.readEntry("Start In Terminal", false))
-        // TODO: start in terminal rather than output view
-#ifdef __GNUC__
-		#warning Implement a Konsole instrumentor
-#endif
         run.setInstrumentor("konsole");
     else
         run.setInstrumentor("default");
+    run.setRunAsUser(group.readEntry("Run As User", QString()));
 
     QStringList compileItems=group.readEntry("Compile Items", QStringList());
     int actionDeps=group.readEntry("BeforeExecute", 1);
@@ -423,6 +445,8 @@ void KDevelop::RunController::registerJob(KJob * job)
         d->stopAction->addAction(stopJobAction);
         connect (stopJobAction, SIGNAL(triggered(bool)), SLOT(slotKillJob()));
 
+        job->setUiDelegate( new KDialogJobUiDelegate() );
+
         d->jobs.insert(job, stopJobAction);
 
         IRunController::registerJob(job);
@@ -555,11 +579,15 @@ void KDevelop::RunJob::start()
 
     m_provider->execute(m_run, this);
 
-    setStandardToolView(IOutputView::RunView);
-    setDelegate(m_controller->delegate());
-    setTitle(m_run.executable().path());
-    setBehaviours( KDevelop::IOutputView::AllowUserClose | KDevelop::IOutputView::AutoScroll );
-    startOutput();
+    // TODO should this really be hard-coded? or should the provider get a say...
+    // Don't show the run output if the process is being run in konsole
+    if (m_run.instrumentor() != "konsole") {
+        setStandardToolView(IOutputView::RunView);
+        setDelegate(m_controller->delegate());
+        setTitle(m_run.executable().path());
+        setBehaviours( KDevelop::IOutputView::AllowUserClose | KDevelop::IOutputView::AutoScroll );
+        startOutput();
+    }
 }
 
 QList< KJob * > KDevelop::RunController::currentJobs() const
@@ -595,6 +623,7 @@ void KDevelop::RunJob::slotFinished(KJob * job)
 
 bool KDevelop::RunJob::doKill()
 {
+    setError(KJob::KilledJobError);
     m_provider->abort(this);
 
     return true;
