@@ -26,6 +26,7 @@
 #include <QTreeView>
 #include <QList>
 #include <QTime>
+#include <QProgressBar>
 #include <KUrl>
 #include "../veritasexport.h"
 
@@ -46,10 +47,12 @@ class ResultsModel;
 class ResultsProxyModel;
 class SelectionManager;
 class VerboseToggle;
+class ToSourceToggle;
 class ResultsWidget;
 class OverlayManager;
 class TestExecutor;
 class Test;
+class TestBar;
 
 /*!
  * \brief The RunnerWindow class defines the Veritas main toolwindow
@@ -96,15 +99,18 @@ public: // Operations
 
     void resetProgressBar() const;
     KDevelop::IProject* selectedProject() const;
+    QString loadedProjectName() const;
 
 Q_SIGNALS:
     void runCompleted() const;
+    void requestReload() const;
 
 public Q_SLOTS:
 
     void showVerboseTestOutput();
     void addProjectToPopup(KDevelop::IProject*);
     void rmProjectFromPopup(KDevelop::IProject*);
+    void openTestSource();
 
 private Q_SLOTS:
 
@@ -194,10 +200,8 @@ private: // Operations
     RunnerWindow(const RunnerWindow&);
     RunnerWindow& operator=(const RunnerWindow&);
     void updateRunText() const;
-    void setRedBar() const;
-    void setGreenBar() const;
 
-    QProgressBar* progressBar() const;
+    TestBar* progressBar() const;
 
 private:
     Ui::RunnerWindow *m_ui;            // QtDesigner main object
@@ -205,15 +209,26 @@ private:
     SelectionManager* m_selection;     // is responsable for the fade-in out selection thingy
     OverlayManager* m_verbose;
     KSelectAction* m_projectPopup;     // a dropdown box to select the 'current' project
-    QMap<KUrl, QAction*> m_project2action;
+    QMap<QString, QAction*> m_project2action;
     QTime m_stopWatch;                 // times test-runs, shown in the gui
     TestExecutor* m_executor;
     mutable bool m_isRunning;
     VerboseToggle* m_verboseToggle;
-    KUrl m_currentProject;
+    ToSourceToggle* m_toSourceToggle;
+    OverlayManager* m_toSource;
+    QString m_currentProject;
     int m_numItemsCompleted;
 };
 
+class TestBar : public QProgressBar
+{
+public:
+    TestBar(QWidget* parent);
+    virtual ~TestBar();
+    void turnGreen();
+    void turnRed();
+};
+
 } // namespace
 
 #endif // VERITAS_RUNNERWINDOW_H
